import pickle
import sys

import pytest


@pytest.fixture
def timings_object(request):
    from pytimings.timer import Timings

    return Timings()


@pytest.fixture
def pickled_timings_object(request, shared_datadir):
    filename = "nested.pickle"
    obj = pickle.loads((shared_datadir / filename).read_bytes())
    obj.extra_data = {}
    return obj


def is_windows_platform():
    return sys.platform == "win32" or sys.platform == "cygwin"


def is_mac_platform():
    return sys.platform == "darwin"



import atexit
import os
import time
from functools import partial
from pathlib import Path
from typing import Optional, Union


def ensure_directory_exists(dirname):
    """create dirname if it doesn't exist"""
    try:
        os.makedirs(dirname)  # noqa: PTH103
    except FileExistsError:
        pass


def output_at_exit(
    output_dir: Optional[Union[str, Path]] = None,
    csv_base="timings",
    timings=None,
    files=True,
    console=True,
) -> None:
    """Register output methods to be executed at Python interpreter exit

    files: set to False to not write csv files to disk
    console: set to False to not display console output for timing sections
    csv_base: filename stem for csv outputs
    output_dir: directory for csv outputs, defaults to current working directory
    """
    from pytimings.timer import global_timings

    timings = timings or global_timings
    if files:
        output_dir = output_dir or os.getcwd()  # noqa: PTH109
        output_dir = Path(output_dir)
        ensure_directory_exists(output_dir)
        output = partial(timings.output_files, output_dir=output_dir, csv_base=csv_base)
        atexit.register(output)
    if console:
        atexit.register(timings.output_console)


def busywait(secs):
    """busywait simulates load, so user time won't be 0 in timings"""
    init_time = time.time()
    while time.time() < init_time + secs:
        pass
    return time.time() - init_time


def generate_example_data(output_dir, number_of_runs=10):
    from pytimings.timer import Timings, scoped_timing

    files = []
    for i in range(1, number_of_runs):
        timings = Timings()
        timings.add_extra_data({"run": i})
        with scoped_timing("linear", timings=timings):
            busywait(number_of_runs / 10 / i)
        with scoped_timing("quadratic", timings=timings):
            busywait(number_of_runs / 10 / i**2)
        files.append(timings.output_files(output_dir=output_dir, csv_base=f"example_speedup_{i:05}"))
    return files



"""This makes sure all our fixtures are available to all tests

Individual test modules MUST NOT import fixtures from `tests.fixtures`,
as this can have strange side effects.
"""

pytest_plugins = [
    "tests.fixtures",
]



from unittest.mock import patch

from pytimings.tools import generate_example_data


def test_generate_example_data_creates_correct_number_of_files():
    with (
        patch("pytimings.timer.Timings") as MockTimings,  # noqa: N806
        patch("pytimings.timer.scoped_timing"),
        patch("pytimings.tools.busywait"),
    ):
        mock_timings = MockTimings.return_value
        mock_timings.output_files.return_value = "mock_file"
        files = generate_example_data("mock_output_dir", number_of_runs=5)
        assert len(files) == 4  # noqa: PLR2004


def test_generate_example_data_handles_zero_runs():
    with (
        patch("pytimings.timer.Timings") as MockTimings,  # noqa: N806
        patch("pytimings.timer.scoped_timing"),
        patch("pytimings.tools.busywait"),
    ):
        mock_timings = MockTimings.return_value
        mock_timings.output_files.return_value = "mock_file"
        files = generate_example_data("mock_output_dir", number_of_runs=0)
        assert len(files) == 0


def test_generate_example_data_creates_files_with_correct_names():
    with (
        patch("pytimings.timer.Timings") as MockTimings,  # noqa: N806
        patch("pytimings.timer.scoped_timing"),
        patch("pytimings.tools.busywait"),
    ):
        mock_timings = MockTimings.return_value
        mock_timings.output_files.side_effect = lambda output_dir, csv_base: f"{csv_base}.csv"
        files = generate_example_data("mock_output_dir", number_of_runs=3)
        assert files == ["example_speedup_00001.csv", "example_speedup_00002.csv"]


def test_generate_example_data_handles_large_number_of_runs():
    with (
        patch("pytimings.timer.Timings") as MockTimings,  # noqa: N806
        patch("pytimings.timer.scoped_timing"),
        patch("pytimings.tools.busywait"),
    ):
        mock_timings = MockTimings.return_value
        mock_timings.output_files.return_value = "mock_file"
        files = generate_example_data("mock_output_dir", number_of_runs=100)
        assert len(files) == 99  # noqa: PLR2004



from io import StringIO

import pytimings
from pytimings.processing import csv_to_dataframe
from pytimings.tools import generate_example_data


def _content(fd):
    # last line is the version field which is ever changing
    # compare files are stored with unix lf and git set to not modify
    return "".join(fd.readlines()[:-1])


def test_output_all_measures(pickled_timings_object, file_regression):
    timing = pickled_timings_object
    with StringIO() as out:
        timing.output_all_measures(out)
        out.seek(0)
        file_regression.check(_content(out))


def test_output_simple(pickled_timings_object, file_regression):
    timing = pickled_timings_object
    with StringIO() as out:
        timing.output_all_measures(out)
        out.seek(0)
        file_regression.check(_content(out))


def test_output_per_rank(pickled_timings_object, file_regression, tmp_path):
    fn = pickled_timings_object.output_files(output_dir=tmp_path, csv_base="per_rank")

    if fn is not None:
        # we're rank 0 and have written the summary file
        file_regression.check(_content(open(fn)))  # noqa: PTH123


def test_csv_to_dataframe(tmpdir):
    files = generate_example_data(tmpdir)
    assert all(f is not None for f in files), files
    assert all(f.is_file() for f in files), files
    frame = csv_to_dataframe(files)
    assert all(frame["pytimings::data::_version"] == pytimings.__version__)



import matplotlib
import pytest

from pytimings.plotting.colors import (
    contrast_ratio,
    discrete_cmap,
    get_colour_palette,
    get_colour_palette_cheat,
    get_hue_vector_rec,
)


@pytest.fixture(scope="function")
def cmap_name():
    name = "indexed"
    yield name
    matplotlib.colormaps.unregister(name)


def test_discrete_cmap_creates_correct_number_of_colors(cmap_name):
    count = 5
    cmap = discrete_cmap(count, name=cmap_name)
    assert len(cmap.colors) == count


def test_discrete_cmap_handles_zero_colors(cmap_name):
    cmap = discrete_cmap(0, name=cmap_name)
    assert len(cmap.colors) == 0


def test_discrete_cmap_uses_default_bg_color(cmap_name):
    cmap = discrete_cmap(3, name=cmap_name)
    assert cmap.colors[0] != (255, 255, 255)


@pytest.mark.skip("blocks indefinitely")
def test_discrete_cmap_uses_custom_bg_color(cmap_name):
    cmap = discrete_cmap(3, bg_color=(0, 0, 0), name=cmap_name)
    assert cmap.colors[0] != (0, 0, 0)


def test_get_colour_palette_cheat_creates_correct_number_of_colors():
    palette = get_colour_palette_cheat(5)
    assert len(palette) == 5  # noqa: PLR2004


def test_get_colour_palette_cheat_handles_zero_colors():
    palette = get_colour_palette_cheat(0)
    assert len(palette) == 0


def test_get_colour_palette_cheat_excludes_filter_colors():
    filter_colors = [(1, 0, 0), (0, 1, 0)]
    palette = get_colour_palette_cheat(5, filter_colors=filter_colors)
    for color in filter_colors:
        assert color not in palette


def test_get_colour_palette_cheat_uses_default_bg_color():
    palette = get_colour_palette_cheat(3)
    for color in palette:
        assert contrast_ratio(color, (1, 1, 1)) < 0.6  # noqa: PLR2004


@pytest.mark.skip("blocks indefinitely")
def test_get_colour_palette_cheat_uses_custom_bg_color():
    palette = get_colour_palette_cheat(3, bg_color=(0, 0, 0))
    for color in palette:
        assert contrast_ratio(color, (0, 0, 0)) < 0.6  # noqa: PLR2004


def test_contrast_ratio_identical_colors():
    assert contrast_ratio((1, 1, 1), (1, 1, 1)) == 1.05 / 1.05


@pytest.mark.skip("contrast ratio broken")
def test_contrast_ratio_black_and_white():
    assert contrast_ratio((0, 0, 0), (1, 1, 1)) == (0.05 + 0.05) / (1.05 + 0.05)


def test_contrast_ratio_color_and_black():
    assert contrast_ratio((0.5, 0.5, 0.5), (0, 0, 0)) > 1


def test_contrast_ratio_color_and_white():
    assert contrast_ratio((0.5, 0.5, 0.5), (1, 1, 1)) < 1


@pytest.mark.skip("contrast ratio broken")
def test_contrast_ratio_handles_low_luminance():
    assert contrast_ratio((0.01, 0.01, 0.01), (0.02, 0.02, 0.02)) > 1


def test_get_colour_palette_creates_correct_number_of_colors():
    palette = get_colour_palette(5)
    assert len(palette) == 5  # noqa: PLR2004


def test_get_colour_palette_handles_zero_colors():
    palette = get_colour_palette(0)
    assert len(palette) == 0


def test_get_colour_palette_uses_correct_hue_values():
    palette = get_colour_palette(3)
    assert all(0 <= color[0] <= 1 for color in palette)


@pytest.mark.skip("broken")
def test_get_colour_palette_uses_correct_saturation_and_value():
    palette = get_colour_palette(3)
    assert all(color[1] == 1 and color[2] == 1 for color in palette)


def test_get_colour_palette_handles_large_number_of_colors():
    size = 100
    palette = get_colour_palette(size)
    assert len(palette) == size


def test_get_hue_vector_rec_creates_correct_hue_vector():
    out = []
    result = get_hue_vector_rec(out, 4, 2)
    assert result == [0.0, 0.5, 0.25, 0.75]


def test_get_hue_vector_rec_handles_single_level():
    out = []
    result = get_hue_vector_rec(out, 2, 1)
    assert result == [0.0, 0.5]


def test_get_hue_vector_rec_handles_zero_amount():
    out = []
    result = get_hue_vector_rec(out, 0, 1)
    assert result == []


def test_get_hue_vector_rec_handles_large_amount():
    out = []
    result = get_hue_vector_rec(out, 8, 3)
    assert result == [0.0, 0.5, 0.25, 0.75, 0.125, 0.625, 0.375, 0.875]


def test_get_hue_vector_rec_handles_non_power_of_two_amount():
    out = []
    result = get_hue_vector_rec(out, 3, 2)
    assert result == [0.0, 0.5, 0.25]



"""Main module."""

from __future__ import annotations

import csv
import functools
import shutil
import sys
import time
from collections import defaultdict, namedtuple
from contextlib import contextmanager
from datetime import timedelta
from io import StringIO
from pathlib import Path

import psutil

import pytimings
from pytimings.tools import ensure_directory_exists

try:
    PERF_COUNTER_FUNCTION = time.perf_counter_ns
    THREAD_TIME_FUNCTION = time.time_ns
    TO_SECONDS_FACTOR = 1e-9
except AttributeError:
    PERF_COUNTER_FUNCTION = time.perf_counter
    try:
        THREAD_TIME_FUNCTION = time.thread_time
    except AttributeError:
        # TODO Log warning
        THREAD_TIME_FUNCTION = time.process_time
    TO_SECONDS_FACTOR = 1

THREAD_TIME = "thread"
WALL_TIME = "wall"
SYS_TIME = "sys"
USER_TIME = "user"

TimingDelta = namedtuple("TimingDelta", [WALL_TIME, SYS_TIME, USER_TIME])


class NoTimerError(Exception):
    def __init__(self, section, timings=None):
        self.section = section
        self.timings = timings or global_timings
        avail = "Available sections: " + ",".join(self.timings._known_timers_map.keys())
        super().__init__(f"trying to access timer for unknown section '{section}'\n{avail}")


class TimingData:
    def __init__(self, name):
        self.name = name
        self._end_resources = None
        self._end_times = None
        self._process = psutil.Process()
        self._start_times = self._get()

    def _get(self):
        ps_times = self._process.cpu_times()
        return {
            USER_TIME: ps_times.user,
            SYS_TIME: ps_times.system,
            WALL_TIME: PERF_COUNTER_FUNCTION(),
            THREAD_TIME: THREAD_TIME_FUNCTION(),
        }

    def stop(self):
        self._end_times = self._get()

    def delta(self):
        delta_times = self._end_times or self._get()

        wall = (delta_times[WALL_TIME] - self._start_times[WALL_TIME]) * TO_SECONDS_FACTOR
        # kernel resource usage already is in seconds
        return TimingDelta(
            wall,
            delta_times[SYS_TIME] - self._start_times[SYS_TIME],
            delta_times[USER_TIME] - self._start_times[USER_TIME],
        )


def _default_timer_dict_entry():
    return (False, None)


class Timings:
    def __init__(self):
        self._commited_deltas: dict[str, TimingDelta] = {}
        self._known_timers_map: dict[str, tuple[bool, TimingData | None]] = defaultdict(_default_timer_dict_entry)
        self.extra_data = dict()
        self.reset()

    def start(self, section_name: str) -> None:
        """set this to begin a named section"""
        if section_name in self._known_timers_map.keys():
            running, data = self._known_timers_map[section_name]
            if running:
                # TODO log info
                return
        self._known_timers_map[section_name] = (True, TimingData(section_name))

    def stop(self, section_name: str | None = None) -> int:
        """stop named section's counter or all of them if section_name is None"""
        if section_name is None:
            for section in self._known_timers_map.keys():
                self.stop(section)
            return
        if section_name not in self._known_timers_map.keys():
            raise NoTimerError(section_name, self)
        self._known_timers_map[section_name] = (
            False,
            self._known_timers_map[section_name][1],
        )  # mark as not running
        timing = self._known_timers_map[section_name][1]
        timing.stop()
        delta = timing.delta()
        if section_name not in self._commited_deltas.keys():
            self._commited_deltas[section_name] = delta
        else:
            previous_delta = self._commited_deltas[section_name]
            new_delta = TimingDelta(*tuple(map(sum, zip(delta, previous_delta))))
            self._commited_deltas[section_name] = new_delta
        return self._commited_deltas[section_name]

    def reset(self, section_name: str | None = None) -> None:
        """set elapsed time back to 0 for a given section or all of them if section_name is None"""
        if section_name is None:
            for section in self._known_timers_map.keys():
                self.reset(section)
            return
        try:
            self.stop(section_name)
        except NoTimerError:
            pass  # not stopping no timer is not an error
        self._commited_deltas[section_name] = TimingDelta(0, 0, 0)

    def walltime(self, section_name: str) -> int:
        """get runtime of section in milliseconds"""
        return self.delta(section_name)[0]

    def add_walltime(self, section_name: str, time: int) -> None:
        self._commited_deltas[section_name] = TimingDelta(time, 0, 0)

    def delta(self, section_name: str) -> dict[str, int]:
        """get the full delta dict"""
        try:
            return self._commited_deltas[section_name]
        except KeyError:
            raise NoTimerError(section_name, self)  # noqa: B904

    def output_files(self, output_dir: Path, csv_base: str) -> Path:
        """output all recorded measures to a csv file"""
        output_dir = Path(output_dir)
        ensure_directory_exists(output_dir)
        outfile = output_dir / f"{csv_base}.csv"
        with outfile.open("w") as out:
            self.output_all_measures(out)
        return outfile

    def output_console(self):
        """outputs walltime only w/o MPI-rank averaging"""
        from rich import box, console, table

        csl = console.Console()
        tbl = table.Table(show_header=True, header_style="bold blue", box=box.SIMPLE_HEAVY)
        tbl.add_column("Extra")
        tbl.add_column("Data")
        for key, value in self.extra_data.items():
            tbl.add_row(key, str(value))
        if len(self.extra_data):
            csl.print(tbl)

        tbl = table.Table(show_header=True, header_style="bold magenta", box=box.SIMPLE_HEAVY)
        tbl.add_column("Section")
        tbl.add_column(
            "Walltime (HH:MM:SS)",
            justify="right",
        )
        for section, delta in self._commited_deltas.items():
            tbl.add_row(section, str(timedelta(seconds=delta[0])))
        if len(self._commited_deltas):
            csl.print(tbl)
        else:
            csl.print("No timings were recorded")

    def output_all_measures(self, out=None) -> None:
        """output all recorded measures

        Outputs average, min, max over all MPI processes associated to mpi_comm
        """
        out = out or sys.stdout
        stash = StringIO()
        csv_file = csv.writer(stash, lineterminator="\n")
        # header
        csv_file.writerow(["section", "value"])

        # threadManager().max_threads()
        csv_file.writerow(["threads", 1])

        for section, delta in self._commited_deltas.items():
            delta = delta._asdict()  # noqa: PLW2901
            wall = delta[WALL_TIME]
            usr = delta[USER_TIME]
            syst = delta[SYS_TIME]
            csv_file.writerows(
                [
                    [f"{section}_usr", usr],
                    [f"{section}_wall", wall],
                    [f"{section}_sys", syst],
                ]
            )
        csv_file.writerows([[f"pytimings::data::{k}", v] for k, v in self.extra_data.items()])
        csv_file.writerow(
            [
                "pytimings::data::_sections",
                "||".join(sorted(self._commited_deltas.keys())),
            ]
        )
        csv_file.writerow(["pytimings::data::_version", pytimings.__version__])
        stash.seek(0)
        shutil.copyfileobj(stash, out)

    def add_extra_data(self, data: [dict]):
        """Use this for something configuration data that makes the csv more informative.

        Data is not displayed with console output.
        """
        self.extra_data.update(data)


global_timings = Timings()


@contextmanager
def scoped_timing(section_name, log_function=None, timings=None, format=""):
    """Start timer on entering block scope, stop it (and optionally output) on exiting.

    The printout will only show walltime for the current scope.
    See :py:func:`pytimings.timer.cummulative_scoped_timing` for a version with cummulative output.
    """
    timings = timings or global_timings
    timings.start(section_name)
    try:
        yield
    finally:
        try:
            previous_wall = timings.walltime(section_name)
        except:  # noqa: E722
            previous_wall = 0
        delta = timings.stop(section_name)
        if log_function:
            log_function(f"Executing {section_name} took {delta.wall-previous_wall:^{format}}s")


@contextmanager
def cummulative_scoped_timing(section_name, log_function=None, timings=None, format=""):
    """Start timer on entering block scope, stop it (and optionally output) on exiting.

    The printout will show the cummulated walltime for all scopes with this section name.
    See :py:func:`pytimings.timer.scoped_timing` for a version with non-cummulative output.
    """
    timings = timings or global_timings
    timings.start(section_name)
    try:
        yield
    finally:
        delta = timings.stop(section_name)
        if log_function:
            log_function(f"Executing {section_name} cummulatively took {delta.wall:^{format}}s")


def function_timer(section_name=None, log_function=None, timings=None):
    def decorator(function):
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            with scoped_timing(
                section_name=section_name or function.__qualname__,
                log_function=log_function,
                timings=timings,
            ):
                return function(*args, **kwargs)

        return wrapper

    return decorator



from collections.abc import Iterable
from logging import getLogger
from pathlib import Path
from typing import Union

import pandas as pd


def csv_to_dataframe(filenames: Iterable[Union[str, Path]], sort=False):
    """Read csv files into a Pandas.DataFrame"""
    if sort:
        filenames = sorted(filenames)
    tbls = [pd.read_csv(fn, index_col=0, header=0, names=[fn]) for fn in filenames]
    timings_cols = [s for s in tbls[0].T.columns if "pytimings::data" not in s]
    dataframe = pd.concat(tbls, axis=1).T
    assert all(dataframe["pytimings::data::_sections"] == dataframe["pytimings::data::_sections"].iloc[0])
    if not all(dataframe["pytimings::data::_version"] == dataframe["pytimings::data::_version"].iloc[0]):
        getLogger("").warning("input csv files created from different pytimings versions")
    for col in timings_cols:
        dataframe[col] = pd.to_numeric(dataframe[col])

    return dataframe



#!/usr/bin/env python

"""Tests for `pytimings` package."""

import pickle
from functools import partial
from tempfile import TemporaryFile

import numpy as np
from click.testing import CliRunner

from pytimings import cli
from pytimings.timer import cummulative_scoped_timing, function_timer, scoped_timing
from pytimings.tools import busywait, output_at_exit

from .fixtures import is_mac_platform, is_windows_platform

_DUMMY_SECTION = "mysection"

DEFAULT_SLEEP_SECONDS = 0.1


default_sleep = partial(busywait, DEFAULT_SLEEP_SECONDS)


def _assert(delta_value, lower=DEFAULT_SLEEP_SECONDS, upper=None):
    if is_windows_platform() or is_mac_platform():
        fuzzy_factor = 0.85
        lower = lower * fuzzy_factor
        if upper:
            upper = upper / fuzzy_factor
    if upper is None:
        assert delta_value > lower
    else:
        assert upper > delta_value > lower


def test_content(timings_object):
    timings_object.start(_DUMMY_SECTION)
    slept = default_sleep()
    timed = timings_object.stop(_DUMMY_SECTION)
    _assert(timed.wall, lower=slept)
    timings_object.stop()
    timings_object.output_all_measures()
    timings_object.reset(_DUMMY_SECTION)
    timings_object.reset()
    timings_object.start(_DUMMY_SECTION)
    slept = default_sleep()
    timed = timings_object.stop(_DUMMY_SECTION)
    _assert(timed.wall, lower=slept, upper=1.1 * slept)
    timings_object.stop()
    timings_object.output_all_measures()


def test_nesting(timings_object):
    scope = partial(scoped_timing, timings=timings_object)
    sleeps = []
    with scope("root_section"):
        sleeps.append(default_sleep())
        with scope("root_section.nested_1"):
            sleeps.append(default_sleep())
            with scope("root_section.nested_1.leaf_section"):
                sleeps.append(default_sleep())
            timed = timings_object.delta("root_section.nested_1.leaf_section")
            _assert(timed.wall, lower=sum(sleeps[-1:]), upper=1.1 * sum(sleeps[-2:]))
        timed = timings_object.delta("root_section.nested_1")
        _assert(timed.wall, lower=sum(sleeps[-2:]), upper=1.1 * sum(sleeps[-3:]))
    timed = timings_object.delta("root_section")
    _assert(timed.wall, lower=sum(sleeps), upper=1.1 * sum(sleeps))


def test_pickling(pickled_timings_object):
    with TemporaryFile() as out:
        dump = partial(pickle.dump, pickled_timings_object, out)
        dump(protocol=0)
        dump(protocol=pickle.HIGHEST_PROTOCOL)
        dump(protocol=pickle.DEFAULT_PROTOCOL)


def test_context(timings_object):
    timings_object.start(_DUMMY_SECTION)
    default_sleep()
    timings_object.stop(_DUMMY_SECTION)
    delta_before = timings_object.delta(_DUMMY_SECTION)
    with scoped_timing(_DUMMY_SECTION, timings=timings_object):
        default_sleep()
    delta_after = timings_object.delta(_DUMMY_SECTION)
    _assert(delta_after.wall, lower=delta_before.wall)
    _assert(delta_after.user, lower=delta_before.user)

    # use the global timings object default
    delta_before = timings_object.delta(_DUMMY_SECTION)
    with scoped_timing(_DUMMY_SECTION):
        default_sleep()
    delta_after = timings_object.delta(_DUMMY_SECTION)
    assert delta_after == delta_before


def test_decorator(timings_object):
    @function_timer(section_name=_DUMMY_SECTION, timings=timings_object)
    def decorated():
        default_sleep()

    assert _DUMMY_SECTION not in timings_object._known_timers_map.keys()
    decorated()
    delta_before = timings_object.delta(_DUMMY_SECTION)
    decorated()
    delta_after = timings_object.delta(_DUMMY_SECTION)
    _assert(delta_after.wall, lower=delta_before.wall)
    _assert(delta_after.user, lower=delta_before.user)


def test_decorator_default_name(timings_object):
    @function_timer(timings=timings_object)
    def my_decorated_name():
        default_sleep()

    class OuterClass:
        @function_timer(timings=timings_object)
        def my_inner_name(self):
            default_sleep()

    section = "test_decorator_default_name.<locals>.my_decorated_name"
    assert section not in timings_object._known_timers_map.keys()
    my_decorated_name()
    delta_before = timings_object.delta(section)
    my_decorated_name()
    delta_after = timings_object.delta(section)
    _assert(delta_after.wall, lower=delta_before.wall)
    _assert(delta_after.user, lower=delta_before.user)

    section = "test_decorator_default_name.<locals>.OuterClass.my_inner_name"
    assert section not in timings_object._known_timers_map.keys()
    obj = OuterClass()
    obj.my_inner_name()
    delta_before = timings_object.delta(section)
    obj.my_inner_name()
    delta_after = timings_object.delta(section)
    _assert(delta_after.wall, lower=delta_before.wall)
    _assert(delta_after.user, lower=delta_before.user)


def test_simple_output(timings_object):
    # simple output with optional format specifier
    with scoped_timing("time", print, timings=timings_object, format=".5f"):
        default_sleep()
    with scoped_timing("a much longer section name", print, timings=timings_object):
        busywait(1)
    scoped_sleepy_time = timings_object.walltime("time")
    timings_object.output_console()

    # add a known walltime to the timings object
    timings_object.add_walltime("sleepy_time", DEFAULT_SLEEP_SECONDS)
    sleepy_time = timings_object.walltime("sleepy_time")
    np.isclose(scoped_sleepy_time, sleepy_time)


def test_cummulative_scoped_timings(timings_object):
    # 'scoped_timing' and 'cummulative_scoped_timing' only differ in terms of their output, but not in terms
    # of their way of storing deltas
    with scoped_timing("time", print, timings=timings_object, format=".5f"):
        default_sleep()
    with scoped_timing("time", print, timings=timings_object, format=".5f"):
        # printout will be default_sleep()
        default_sleep()

    with cummulative_scoped_timing("another time", print, timings=timings_object, format=".5f"):
        default_sleep()
    with cummulative_scoped_timing("another time", print, timings=timings_object, format=".5f"):
        # printout will be 2 * default_sleep()
        default_sleep()

    # but this doesn't matter for the objects
    np.isclose(timings_object.walltime("time"), timings_object.walltime("another time"))


def test_atexit(timings_object):
    """This only tests if the setup function can be called"""
    output_at_exit(timings=timings_object)


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert "pytimings.cli.main" in result.output
    help_result = runner.invoke(cli.main, ["--help"])
    assert help_result.exit_code == 0
    assert "--help  Show this message and exit." in help_result.output



#!/usr/bin/env python3

"""The setup script."""

import os
import sys

from setuptools import find_packages, setup

sys.path.append(os.path.dirname(__file__))  # noqa: PTH120
import dependencies
import versioneer

tests_require = dependencies.ci_requires
install_requires = dependencies.install_requires
setup_requires = dependencies.setup_requires()
install_suggests = dependencies.install_suggests

with open("README.md") as readme_file:  # noqa: PTH123
    readme = readme_file.read()

setup(
    author="René Fritze",
    author_email="coding@fritze.me",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    description="MPI Aware section timings",
    entry_points={
        "console_scripts": [
            "pytimings=pytimings.cli:main",
        ],
    },
    tests_require=tests_require,
    install_requires=install_requires,
    extras_require=dependencies.extras(),
    license="BSD license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="pytimings",
    name="pytimings",
    packages=find_packages(include=["pytimings", "pytimings.*"]),
    url="https://github.com/renefritze/pytimings",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    zip_safe=False,
)
